from urllib.parse import urljoin

import requests
from requests import Response

from biolib.typing_utils import Dict, Optional
from biolib.biolib_api_client import BiolibApiClient as DeprecatedApiClient


class ApiClient:

    def get(self, url: str, params: Dict[str, str]) -> Response:
        deprecated_api_client = DeprecatedApiClient.get()

        access_token: Optional[str] = None
        if deprecated_api_client.is_signed_in:
            deprecated_api_client.refresh_access_token()
            access_token = deprecated_api_client.access_token

        base_api_url = urljoin(deprecated_api_client.base_url, '/api/')
        absolute_url = urljoin(base_api_url, url.strip('/') + '/')

        response = requests.get(
            headers={'Authorization': f'Bearer {access_token}'} if access_token else None,
            params=params,
            url=absolute_url,
        )
        response.raise_for_status()
        return response



import asyncio
import logging
from typing import Any, Awaitable

logger = logging.getLogger(__name__)


class TaskQueue:
    def __init__(self, loop: asyncio.AbstractEventLoop = None):
        self.loop = loop or asyncio.get_event_loop()
        fut = self.loop.create_future()
        fut.set_result(None)
        self._task_chain = fut

    def post_task(self, task: Awaitable[Any]) -> Awaitable[Any]:
        async def run_awaitable(prev) -> None:
            try:
                await prev
                return await task
            except Exception as e:
                logger.error(f'Exception while evaluating task queue: {e}')

        self._task_chain = self.loop.create_task(run_awaitable(self._task_chain))
        return self._task_chain



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Utility functions."""

import gc
import socket


def get_free_port() -> int:
    """Get free port."""
    sock = socket.socket()
    sock.bind(('localhost', 0))
    port = sock.getsockname()[1]
    sock.close()
    del sock
    gc.collect()
    return port



import asyncio
import logging
from typing import Any, AsyncIterable, Callable, Iterable, Optional, Union

from websockets import Data, WebSocketClientProtocol, connect

logger = logging.getLogger(__name__)


class WebsocketTransport:
    def __init__(self, ws: WebSocketClientProtocol):
        self.onmessage: Optional[Callable[[str], Any]] = None
        self.onclose: Optional[Callable[[], Any]] = None
        self.ws = ws

    @classmethod
    async def create(cls, uri: str, loop: asyncio.AbstractEventLoop = None) -> 'WebsocketTransport':
        return cls(
            await connect(
                uri=uri,
                # chrome doesn't respond to pings
                # todo: remove note after websockets release
                # waiting on websockets to release new version where ping_interval is typed correctly
                ping_interval=None,  # type: ignore
                max_size=256 * 1024 * 1024,  # 256Mb
                loop=loop,
                close_timeout=5,
                # todo check if speed is affected
                # note: seems to work w/ compression
                compression=None,
            )
        )

    async def send(self, message: Union[Data, Iterable[Data], AsyncIterable[Data]]) -> None:
        await self.ws.send(message)

    async def close(self, code: int = 1000, reason: str = '') -> None:
        logger.debug(f'Disposing connection: code={code} reason={reason}')
        await self.ws.close(code=code, reason=reason)
        if self.onclose:
            await self.onclose()

    async def recv(self) -> Data:
        data = await self.ws.recv()
        if self.onmessage and data:
            await self.onmessage(data)
        return data



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['biolib',
 'biolib.api',
 'biolib.app',
 'biolib.biolib_api_client',
 'biolib.biolib_binary_format',
 'biolib.biolib_docker_client',
 'biolib.biolib_singularity_client',
 'biolib.compute_node',
 'biolib.compute_node.cloud_utils',
 'biolib.compute_node.enclave',
 'biolib.compute_node.job_worker',
 'biolib.compute_node.job_worker.executors',
 'biolib.compute_node.job_worker.executors.remote',
 'biolib.compute_node.job_worker.executors.tars',
 'biolib.compute_node.webserver',
 'biolib.jobs',
 'biolib.pyppeteer.docs',
 'biolib.pyppeteer.pyppeteer',
 'biolib.pyppeteer.pyppeteer.connection',
 'biolib.pyppeteer.pyppeteer.frame',
 'biolib.pyppeteer.pyppeteer.models',
 'biolib.pyppeteer.utils',
 'biolib.templates',
 'biolib.user',
 'biolib.utils',
 'biolib.validators']

package_data = \
{'': ['*'],
 'biolib': ['biolib-js/*', 'pyppeteer/*', 'pyppeteer/.circleci/*'],
 'biolib.pyppeteer.docs': ['_static/*', '_templates/*']}

install_requires = \
['aenum>=2.2.3,<3.0.0',
 'appdirs>=1.4.3,<2.0.0',
 'cbor2==5.2.0',
 'certifi>=2019.11.28',
 'cose==0.9.dev2',
 'cryptography>=3.1,<36.0.2',
 'docker==4.4.1',
 'flask-cors==3.0.10',
 'flask==2.0.1',
 'gunicorn==20.1.0',
 'importlib-metadata>=1.6.1',
 'nest_asyncio>=1.4.0,<1.5.2',
 'ordered_set>=4.0.1,<5.0.0',
 'pyOpenSSL==19.1.0',
 'pycryptodome==3.9.9',
 'pyee>=7.0.1,<8.0.0',
 'pyjwt==2.3.0',
 'pyyaml>=5.3.1',
 'requests>=2.25.1',
 'spython>=0.1.18',
 'termcolor==1.1.0',
 'tqdm>=4.42.1,<5.0.0',
 'websockets>=8.0']

extras_require = \
{':python_version < "3.8"': ['typing_extensions==3.10.0',
                             'typing_inspect>=0.5.0,<0.6.0']}

entry_points = \
{'console_scripts': ['biolib = biolib:call_cli']}

setup_kwargs = {
    'name': 'pybiolib',
    'version': '1.0.1843',
    'description': 'BioLib Python Client',
    'long_description': "# PyBioLib\n\nPyBioLib is a Python package for running BioLib applications from Python scripts and the command line.\n\n### Python Example\n```python\n# pip3 install -U pybiolib\nimport biolib\nsamtools = biolib.load('samtools/samtools')\nprint(samtools.cli(args='--help'))\n```\n\n### Command Line Example\n```bash\npip3 install -U pybiolib\nbiolib run samtools/samtools --help\n```\n",
    'author': 'biolib',
    'author_email': 'hello@biolib.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/biolib',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
